package io.papermc.hangar.controller.extras.pagination.filters.projects;

import io.papermc.hangar.controller.extras.pagination.Filter;
import io.papermc.hangar.controller.extras.pagination.filters.projects.ProjectCategoryFilter.ProjectCategoryFilterInstance;
import io.papermc.hangar.model.common.projects.Category;
import org.jdbi.v3.core.statement.SqlStatement;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.ConversionService;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.NativeWebRequest;

import java.util.Arrays;
import java.util.Set;

@Component
public class ProjectCategoryFilter implements Filter<ProjectCategoryFilterInstance> {

    private final ConversionService conversionService;

    @Autowired
    public ProjectCategoryFilter(ConversionService conversionService) {
        this.conversionService = conversionService;
    }

    @Override
    public Set<String> getQueryParamNames() {
        return Set.of("category");
    }

    @Override
    public String getDescription() {
        return "A category to filter for";
    }

    @NotNull
    @Override
    public ProjectCategoryFilterInstance create(NativeWebRequest webRequest) {
        return new ProjectCategoryFilterInstance(conversionService.convert(webRequest.getParameterValues(getSingleQueryParam()), Category[].class));
    }

    static class ProjectCategoryFilterInstance implements FilterInstance {

        private final Category[] categories;

        public ProjectCategoryFilterInstance(Category[] categories) {
            this.categories = categories;
        }

        @Override
        public void createSql(StringBuilder sb, SqlStatement<?> q) {
            sb.append(" AND ").append("p.category").append(" IN (");
            for (int i = 0; i < categories.length; i++) {
                sb.append(":__category__").append(i);
                if (i + 1 != categories.length) {
                    sb.append(",");
                }
                q.bind("__category__" + i, categories[i]);
            }
            sb.append(")");
        }

        @Override
        public String toString() {
            return "ProjectCategoryFilterInstance{" +
                    "categories=" + Arrays.toString(categories) +
                    '}';
        }
    }
}
